using System;
using System.Collections;
using System.Web;
using System.Web.UI.WebControls;
using GBPVR.Public;

namespace gbweb.classes
{
	/// <summary>
	/// Summary description for ProgrammeDisplay.
	/// </summary>
	public class ProgrammeDisplay : IDisposable
	{

		public ProgrammeDisplay()
		{
			if (getZapTools())
			{
				ZapTools.OpenZapDB();
			}
		}

		#region IDisposable Members

		public void Dispose()
		{
			if (getZapTools())
			{
				//Close the ZapTools DB
				ZapTools.CloseZapDB();
			}
		}

		#endregion

		#region Zap Detection
		private static bool zapEPG;
		private static bool zapEPGLoaded = false;
		private static bool getZapEPG()
		{
			if (!zapEPGLoaded)
			{
				zapEPG = ZapTools.checkZapEPG();
				zapEPGLoaded = true;
			}
			return zapEPG;
		}

		private static bool zapTools;
		private static bool zapToolsLoaded = false;
		private static bool getZapTools()
		{
			if (!zapToolsLoaded)
			{
				zapTools = ZapTools.Initialize();
				zapToolsLoaded = true;
			}
			return zapTools;
		}
		#endregion

		public void FillProgrammeDisplay(HttpServerUtility Server, TableCell colShow, Programme programme, ScheduledRecording scheduledRecording, bool highlightRecordedShows)
		{
			Settings guideParams = Global.Settings;
			string title = string.Empty;
			string subtitle = string.Empty;
			string href = null;
			string onclick = string.Empty;
			bool externallinks = true;
			string description = string.Empty;
			bool showAsRecording = false;

			if (scheduledRecording != null)
			{
				switch (scheduledRecording.getRecordingType())
				{
					case ScheduledRecording.TYPE_RECORD_ONCE:
						if (programme == null) 
						{
							title = "Manual Recording";
							externallinks = false;
						}
						break;
					default:
						title = scheduledRecording.getFileName();
						href = "SearchResults.aspx?matchTitle=on&textKeyWord=" + Server.UrlEncode(title);
						programme = null;
						break;
				}
				if (highlightRecordedShows) showAsRecording = true;
			}

			if (programme != null)
			{

				//Check to see if the user uses the Zap2it EPG
				bool zapEPG = getZapEPG();
			
				//Check to see if the ZapTools DB is available
				bool zapToolsEnabled = getZapTools();

				title = programme.getTitle();
				string extralinks = string.Empty;
				const string spacer = "<br/><img height=\"4\" src=\"public/t.gif\" border=\"0\"><br/>";

				// Show the programme description for all EPG Users
				if (guideParams.showDescription)
				{
					description = programme.getDescription();
					// If the description is found space it down two lines from the show name and title
					if (description.Length > 0)
					{
						description = spacer + description;
					}
					subtitle = programme.getSubTitle();
					if (subtitle.Length > 0)
					{
						// If there's no description, then add the subtitle as the description
						if (description.Length == 0)
						{
							description = spacer + subtitle;
						}
					}
				}
				// If you are not displaying the description but there is a subtitle then set the variable title to SubTitle
				else if (guideParams.showSubtitle)
				{
					subtitle = programme.getSubTitle();
				}
			
				// Add a double line break if there are other things to display below the description
				if ((guideParams.showGenre != "noGenre") || (zapEPG && guideParams.showRating) || (zapToolsEnabled && (guideParams.showRepeat || guideParams.showCredits)))
				{
					description += spacer + " ";
				}

				// If the user is a Zap2it EPG user pull the rating and genre from the native GBPVR database
				if (zapEPG)
				{
					// Retrieve the rating for the show
					if (guideParams.showRating)
					{
						string rating = programme.getRating();
						//Since this is the first item being added to the variable extalinks we don't need to add line breaks since it was taken care of by
						if (rating != null && rating.Length > 0)
						{
							extralinks += "Rated: " + programme.getRating();
						}
					}
				}

				// Add the show Genre to the line
				if (guideParams.showGenre != "noGenre")
				{
					ArrayList genreCheck = programme.getGenreList();
					for (int unknownPos = genreCheck.BinarySearch("unknown", new CaseInsensitiveComparer()); unknownPos > -1;
					         unknownPos = genreCheck.BinarySearch("unknown", new CaseInsensitiveComparer()))
					{
						genreCheck.RemoveAt(unknownPos);
					}
					for (int unknownPos = genreCheck.BinarySearch(string.Empty, new CaseInsensitiveComparer()); unknownPos > -1;
						unknownPos = genreCheck.BinarySearch(string.Empty, new CaseInsensitiveComparer()))
					{
						genreCheck.RemoveAt(unknownPos);
					}
					if (genreCheck.Count > 0)
					{
						//If another extralink was previously added we want to single space this one down otherwise no extra line is needed
						if (extralinks.Length > 0) extralinks += "<br>";
						if (guideParams.showGenre == "allGenre")
						{
							extralinks += "Genre" + (genreCheck.Count > 1 ? "s" : string.Empty) + ": " + string.Join(", ", (string[])genreCheck.ToArray(typeof(string)));
						}
						else
						{
							extralinks += "Genre: " + genreCheck[0];
						}
					}
				}

				// If the user has ZapTools append additional information to the description when found
				if (zapToolsEnabled)
				{
					string programUniqueIdentifier = programme.getUniqueProgrammeIdentifier();
	
					if (guideParams.showAirDate || guideParams.showStarRating)
					{
						// Pull the Airdate of the Show or the Release Year of the movie and the Start Rating for Movie
						string[] programInfo = ZapTools.GetProgramInfo(programUniqueIdentifier);

						if (programInfo.Length > 0)
						{
							if (programUniqueIdentifier.StartsWith("MV"))
							{
								if ((guideParams.showAirDate) && (programInfo[0] != "0"))
								{
									// If another extralink was previously added we want to single space this one down otherwise no extra line is needed
									if (extralinks.Length > 0) extralinks += "<br>";
									extralinks += " Movie Release Year: "+ programInfo[0].ToString();
								}
								if ((guideParams.showStarRating) && (programInfo[2] != "none"))
								{
									// If another extralink was previously added we want to single space this one down otherwise no extra line is needed
									if (extralinks.Length > 0) extralinks += "<br>";
									extralinks += " Star Rating: "+ programInfo[2];
								}
							}
							else
							{
								if ((guideParams.showAirDate) && (programInfo[1] != "none"))
								{
									// If another extralink was previously added we want to single space this one down otherwise no extra line is needed
									if (extralinks.Length > 0) extralinks += "<br>";
									extralinks += " Original Air Date: "+ programInfo[1];
								}
							}
						}
					}
							
					// Check the ZapTools supplemental database to see if the show is a repeat
					if (guideParams.showRepeat)
					{
						if (ZapTools.IsRepeat(programUniqueIdentifier))
						{
							// If show is a repeat tag an indicator at the end of the description indicating repeat
							// If another extralink was previously added we want to single space this one down otherwise no extra line is needed
							if (extralinks.Length > 0) extralinks += "<br>";
							extralinks += "<b> (Repeat)</b>";
						}
					}

					// Provide a link to see Cast information
					if (guideParams.showCredits)
					{
						// If another extralink was previously added we want to single space this one down otherwise no extra line is needed
						if (extralinks.Length > 0) extralinks += "<br>";
						extralinks += "(<a href=\"Credits.aspx?id=" + programme.getOID() + "\" onclick=\"EditPop2(this.href,'Add1');return false;\">Credits</a>)";
					}
				}

				href = "Details.aspx?id=" + programme.getOID();
				if (scheduledRecording != null)
				{
					if (highlightRecordedShows) showAsRecording = true;
					href += "&rid=" + scheduledRecording.getOID();
				}
				onclick = " onclick=\"EditPop(this.href,'Add2');return false;\"";

				description = description + extralinks;
			}

			// Add external links (these must be added in reverse order)
			// Show the show link to Netflix for all EPG Users
			if (externallinks && guideParams.showNetflix)
			{
				description = " (<a href=\"http://www.netflix.com/Search?dtl=1&type=title&v1=" + Server.UrlEncode(title) + "\" target=\"_blank\">Netflix</a>)" + description;
			}
			// Show the show link to IMDB for all EPG Users
			if (externallinks && guideParams.showIMDB)
			{
				description = " (<a href=\"http://www.imdb.com/find?s=tt&q=" + Server.UrlEncode(title) + "\" target=\"_blank\">IMDB</a>)" + description;
			}

			string spanclass;
			if (showAsRecording)
			{
				int showStatus = scheduledRecording.getRecordingStatus();
				switch (showStatus)
				{
					case ScheduledRecording.STATUS_PENDING:
						colShow.CssClass = "pending";
						break;
					case ScheduledRecording.STATUS_IN_PROGRESS:
						colShow.CssClass = "inprogress";
						break;
					case ScheduledRecording.STATUS_COMPLETED:
						colShow.CssClass = "available";
						break;
					case ScheduledRecording.STATUS_COMPLETED_WITH_ERROR:
						colShow.CssClass = "failed";
						break;
					case ScheduledRecording.STATUS_CONFLICT:
						colShow.CssClass = "conflict";
						break;
					default:
						colShow.CssClass = "recorded";
						break;
				}
				colShow.CssClass = "listingr " + colShow.CssClass;
				spanclass = "listdescr";
			}
			else
			{
				colShow.CssClass = "listing";
				spanclass = "listdesc";
			}

			if (subtitle.Length > 0) title += ": <span class=\"subtitle\">" +  subtitle + "</span>";
			title = "<b>" + title + "</b>";
			if (href != null) title = "<a href=\"" + href + "\"" + onclick + ">" + title + "</a>";
			if (description.Length > 0) title += "<span class=\"" + spanclass + "\">" + description + "</span>";
			colShow.Text = title;
		}
	}
}
